@@ -1,5 +1,6 @@
 /* This file is part of Step.
    Copyright (C) 2007 Vladimir Kuznetsov <ks.vladimir@gmail.com>
+   Copyright (C) 2014 Inge Wallin        <inge@lysator.liu.se>
 
    Step is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,7 +18,6 @@
 */
 
 #include "toolgraphics.h"
-#include "toolgraphics.moc"
 
 #include "ui_configure_graph.h"
 #include "ui_configure_meter.h"
@@ -47,27 +47,33 @@
 #include <QLCDNumber>
 #include <QFocusEvent>
 #include <QApplication>
+#include <QAction>
+#include <QUrl>
+#include <QDialog>
+#include <QDialogButtonBox>
+#include <QAbstractButton>
+#include <QPushButton>
+#include <QVBoxLayout>
+
 #include <KToolBar>
 #include <KPlotWidget>
 #include <KPlotObject>
 #include <KPlotPoint>
 #include <KPlotAxis>
-#include <KDialog>
 #include <KFileDialog>
 #include <KProgressDialog>
-#include <KAction>
 #include <KToggleAction>
 #include <KFontAction>
 #include <KFontSizeAction>
 #include <KColorDialog>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KMessageBox>
 #include <KInputDialog>
 #include <KIO/NetAccess>
-#include <KDebug>
 
 #include <float.h>
 
+
 StepCore::Vector2d WidgetVertexHandlerGraphicsItem::value()
 {
     double s = currentViewScale();
@@ -276,8 +282,8 @@ void NoteTextEdit::focusInEvent(QFocusEvent *event)
 
 void NoteTextEdit::focusOutEvent(QFocusEvent *event)
 {
-    kDebug() << event->reason() << endl;
-    kDebug() << QApplication::focusWidget()->metaObject()->className() << endl;
+    qDebug() << event->reason() << endl;
+    qDebug() << QApplication::focusWidget()->metaObject()->className() << endl;
 
     QObject* f = QApplication::focusWidget();
     if(f == this) f = NULL;
@@ -380,22 +386,22 @@ NoteGraphicsItem::NoteGraphicsItem(StepCore::Item* item, WorldModel* worldModel)
     if(_toolBar->layout()) _toolBar->layout()->setSpacing(0);
     //_toolBar->setStyleSheet(".KToolBar {margin: 0px; border-width: 0px; padding: 0px; }");
 
-    _actionColor = new KAction(KIcon(), i18n("&Color"), _toolBar);
+    _actionColor = new QAction(QIcon(), i18n("&Color"), _toolBar);
 
-    _actionBold = new KToggleAction(KIcon("format-text-bold"), i18n("&Bold"), _toolBar);
+    _actionBold = new KToggleAction(QIcon::fromTheme("format-text-bold"), i18n("&Bold"), _toolBar);
     _actionBold->setShortcut(Qt::CTRL + Qt::Key_B);
-    _actionItalic = new KToggleAction(KIcon("format-text-italic"), i18n("&Italic"), _toolBar);
+    _actionItalic = new KToggleAction(QIcon::fromTheme("format-text-italic"), i18n("&Italic"), _toolBar);
     _actionItalic->setShortcut(Qt::CTRL + Qt::Key_I);
-    _actionUnderline = new KToggleAction(KIcon("format-text-underline"), i18n("&Underline"), _toolBar);
+    _actionUnderline = new KToggleAction(QIcon::fromTheme("format-text-underline"), i18n("&Underline"), _toolBar);
     _actionUnderline->setShortcut(Qt::CTRL + Qt::Key_U);
 
-    _actionAlignLeft = new KToggleAction(KIcon("format-justify-left"), i18n("Align &Left"), _toolBar);
+    _actionAlignLeft = new KToggleAction(QIcon::fromTheme("format-justify-left"), i18n("Align &Left"), _toolBar);
     _actionAlignLeft->setShortcut(Qt::CTRL + Qt::Key_L);
-    _actionAlignCenter = new KToggleAction(KIcon("format-justify-center"), i18n("Align C&enter"), _toolBar);
+    _actionAlignCenter = new KToggleAction(QIcon::fromTheme("format-justify-center"), i18n("Align C&enter"), _toolBar);
     _actionAlignCenter->setShortcut(Qt::CTRL + Qt::Key_E);
-    _actionAlignRight = new KToggleAction(KIcon("format-justify-right"), i18n("Align &Right"), _toolBar);
+    _actionAlignRight = new KToggleAction(QIcon::fromTheme("format-justify-right"), i18n("Align &Right"), _toolBar);
     _actionAlignRight->setShortcut(Qt::CTRL + Qt::Key_R);
-    _actionAlignJustify = new KToggleAction(KIcon("format-justify-fill"), i18n("Align &Justify"), _toolBar);
+    _actionAlignJustify = new KToggleAction(QIcon::fromTheme("format-justify-fill"), i18n("Align &Justify"), _toolBar);
     _actionAlignJustify->setShortcut(Qt::CTRL + Qt::Key_J);
 
     _actionAlign = new KSelectAction(i18n("&Align"), _toolBar);
@@ -410,11 +416,11 @@ NoteGraphicsItem::NoteGraphicsItem(StepCore::Item* item, WorldModel* worldModel)
     _actionFont = new KFontAction(i18n("&Font"), _toolBar);
     _actionFontSize = new KFontSizeAction(i18n("Font &Size"), _toolBar);
 
-    _actionInsertImage = new KAction(KIcon("insert-image"), i18n("Insert &Image"), _toolBar);
+    _actionInsertImage = new QAction(QIcon::fromTheme("insert-image"), i18n("Insert &Image"), _toolBar);
 #ifdef __GNUC__
 #warning Select right icon here
 #endif
-    _actionInsertFormula = new KAction(KIcon("application-vnd.oasis.opendocument.formula"),
+    _actionInsertFormula = new QAction(QIcon::fromTheme("application-vnd.oasis.opendocument.formula"),
                                     i18n("Insert &Formula"), _toolBar);
 
     connect(_actionColor, SIGNAL(triggered(bool)), this, SLOT(formatColor()));
@@ -631,7 +637,7 @@ void NoteGraphicsItem::cursorPositionChanged()
 
 void NoteGraphicsItem::insertImage()
 {
-    KUrl url = KFileDialog::getOpenUrl(KUrl(), "image/png image/jpeg", _widget);
+    QUrl url = KFileDialog::getOpenUrl(QUrl(), "image/png image/jpeg", _widget);
     if(url.isEmpty()) return;
 
     QString tmpFileName;
@@ -1081,8 +1087,8 @@ void GraphMenuHandler::populateMenu(QMenu* menu, KActionCollection* actions)
     _confDialog = 0;
     _confChanged = false;
 
-    menu->addAction(KIcon("edit-clear"), i18n("Clear graph"), this, SLOT(clearGraph()));
-    menu->addAction(KIcon("configure"), i18n("Configure graph..."), this, SLOT(configureGraph()));
+    menu->addAction(QIcon::fromTheme("edit-clear"), i18n("Clear graph"), this, SLOT(clearGraph()));
+    menu->addAction(QIcon::fromTheme("configure"), i18n("Configure graph..."), this, SLOT(configureGraph()));
     menu->addSeparator();
     ItemMenuHandler::populateMenu(menu, actions);
 }
@@ -1098,13 +1104,26 @@ void GraphMenuHandler::configureGraph()
         _worldModel->simulationStop();
 
     _confChanged = false;
-    _confDialog = new KDialog(); // XXX: parent?
+    _confDialog = new QDialog(); // XXX: parent?
     
-    _confDialog->setCaption(i18n("Configure graph"));
-    _confDialog->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
+    _confDialog->setWindowTitle(i18n("Configure graph"));
+    _buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
+				      | QDialogButtonBox::Cancel
+				      | QDialogButtonBox::Apply);
+    QWidget *mainWidget = new QWidget();
+    QVBoxLayout *mainLayout = new QVBoxLayout;
+    _confDialog->setLayout(mainLayout);
+    mainLayout->addWidget(mainWidget);
+
+    QPushButton *okButton = _buttonBox->button(QDialogButtonBox::Ok);
+    okButton->setDefault(true);
+    okButton->setShortcut(Qt::CTRL | Qt::Key_Return);
+    _confDialog->connect(_buttonBox, SIGNAL(accepted()), _confDialog, SLOT(accept()));
+    _confDialog->connect(_buttonBox, SIGNAL(rejected()), _confDialog, SLOT(reject()));
+    mainLayout->addWidget(_buttonBox);
 
     _confUi = new Ui::WidgetConfigureGraph;
-    _confUi->setupUi(_confDialog->mainWidget());
+    _confUi->setupUi(mainWidget);
 
     _confUi->dataSourceX->setDataSource(_worldModel,
                     graph()->objectX(), graph()->propertyX(), graph()->indexX());
@@ -1131,10 +1150,9 @@ void GraphMenuHandler::configureGraph()
     _confUi->checkBoxShowLines->setChecked(graph()->showLines());
     _confUi->checkBoxShowPoints->setChecked(graph()->showPoints());
 
-    _confDialog->enableButtonApply(false);
+    _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 
-    connect(_confDialog, SIGNAL(applyClicked()), this, SLOT(confApply()));
-    connect(_confDialog, SIGNAL(okClicked()), this, SLOT(confApply()));
+    connect(_buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(confApply(QAbstractButton*)));
 
     connect(_confUi->dataSourceX, SIGNAL(dataSourceChanged()), this, SLOT(confChanged()));
     connect(_confUi->dataSourceY, SIGNAL(dataSourceChanged()), this, SLOT(confChanged()));
@@ -1153,8 +1171,13 @@ void GraphMenuHandler::configureGraph()
     delete _confUi; _confUi = 0;
 }
 
-void GraphMenuHandler::confApply()
+void GraphMenuHandler::confApply(QAbstractButton *button)
 {
+    if (_buttonBox->button(QDialogButtonBox::Apply) != button
+	&& _buttonBox->button(QDialogButtonBox::Ok) != button) {
+	return;
+    }
+
     Q_ASSERT(_confUi && _confDialog);
 
     // XXX: check for actual change ?
@@ -1204,7 +1227,7 @@ void GraphMenuHandler::confChanged()
 {
     Q_ASSERT(_confUi && _confDialog);
     _confChanged = true;
-    _confDialog->enableButtonApply(true);
+    _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
 }
 
 void GraphMenuHandler::clearGraph()
@@ -1263,8 +1286,8 @@ void MeterGraphicsItem::worldDataChanged(bool dynamicOnly)
     if(!dynamicOnly) {
         viewScaleChanged();
 
-        if(meter()->digits() != _lcdNumber->numDigits())
-            _lcdNumber->setNumDigits(meter()->digits());
+        if(meter()->digits() != _lcdNumber->digitCount())
+            _lcdNumber->setDigitCount(meter()->digits());
 
         QString units = meter()->units();
         if(units != _labelUnits->text()) {
@@ -1292,7 +1315,7 @@ void MeterMenuHandler::populateMenu(QMenu* menu, KActionCollection* actions)
     _confDialog = 0;
     _confChanged = false;
 
-    menu->addAction(KIcon("configure"), i18n("Configure meter..."), this, SLOT(configureMeter()));
+    menu->addAction(QIcon::fromTheme("configure"), i18n("Configure meter..."), this, SLOT(configureMeter()));
     menu->addSeparator();
     ItemMenuHandler::populateMenu(menu, actions);
 }
@@ -1308,13 +1331,26 @@ void MeterMenuHandler::configureMeter()
         _worldModel->simulationStop();
 
     _confChanged = false;
-    _confDialog = new KDialog(); // XXX
-    
-    _confDialog->setCaption(i18n("Configure meter"));
-    _confDialog->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
+
+    _confDialog = new QDialog(); // XXX
+    _confDialog->setWindowTitle(i18n("Configure meter"));
+    _buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
+				      | QDialogButtonBox::Cancel
+				      | QDialogButtonBox::Apply);
+    QWidget *mainWidget = new QWidget();
+    QVBoxLayout *mainLayout = new QVBoxLayout;
+    _confDialog->setLayout(mainLayout);
+    mainLayout->addWidget(mainWidget);
+
+    QPushButton *okButton = _buttonBox->button(QDialogButtonBox::Ok);
+    okButton->setDefault(true);
+    okButton->setShortcut(Qt::CTRL | Qt::Key_Return);
+    _confDialog->connect(_buttonBox, SIGNAL(accepted()), _confDialog, SLOT(accept()));
+    _confDialog->connect(_buttonBox, SIGNAL(rejected()), _confDialog, SLOT(reject()));
+    mainLayout->addWidget(_buttonBox);
 
     _confUi = new Ui::WidgetConfigureMeter;
-    _confUi->setupUi(_confDialog->mainWidget());
+    _confUi->setupUi(mainWidget);
 
     _confUi->dataSource->setDataSource(_worldModel,
                 meter()->object(), meter()->property(), meter()->index());
@@ -1323,8 +1359,7 @@ void MeterMenuHandler::configureMeter()
                 new QIntValidator(0, 100, _confUi->lineEditDigits));
     _confUi->lineEditDigits->setText(QString::number(meter()->digits()));
 
-    connect(_confDialog, SIGNAL(applyClicked()), this, SLOT(confApply()));
-    connect(_confDialog, SIGNAL(okClicked()), this, SLOT(confApply()));
+    connect(_buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(confApply(QAbstractButton*)));
 
     connect(_confUi->dataSource, SIGNAL(dataSourceChanged()), this, SLOT(confChanged()));
     connect(_confUi->lineEditDigits, SIGNAL(textEdited(const QString&)), this, SLOT(confChanged()));
@@ -1335,8 +1370,13 @@ void MeterMenuHandler::configureMeter()
     delete _confUi; _confUi = 0;
 }
 
-void MeterMenuHandler::confApply()
+void MeterMenuHandler::confApply(QAbstractButton *button)
 {
+    if (_buttonBox->button(QDialogButtonBox::Apply) != button
+	&& _buttonBox->button(QDialogButtonBox::Ok) != button) {
+	return;
+    }
+
     Q_ASSERT(_confUi && _confDialog);
 
     // XXX: check for actual change ?
@@ -1360,7 +1400,7 @@ void MeterMenuHandler::confChanged()
 {
     Q_ASSERT(_confUi && _confDialog);
     _confChanged = true;
-    _confDialog->enableButtonApply(true);
+    _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
 }
 
 ////////////////////////////////////////////////////
@@ -1390,8 +1430,8 @@ ControllerGraphicsItem::ControllerGraphicsItem(StepCore::Item* item, WorldModel*
     layout->addWidget(_labelMax, 0, 2, 1, 1);
     layout->addWidget(_labelSource, 1, 0, 1, 3);
 
-    _incAction = new KAction(i18n("Increase value"), _widget);
-    _decAction = new KAction(i18n("Decrease value"), _widget);
+    _incAction = new QAction(i18n("Increase value"), _widget);
+    _decAction = new QAction(i18n("Decrease value"), _widget);
 
     connect(_incAction, SIGNAL(triggered(bool)), this, SLOT(incTriggered()));
     connect(_decAction, SIGNAL(triggered(bool)), this, SLOT(decTriggered()));
@@ -1463,12 +1503,12 @@ void ControllerGraphicsItem::worldDataChanged(bool dynamicOnly)
 
         if(_incShortcut != controller()->increaseShortcut()) {
             _incShortcut = controller()->increaseShortcut();
-            _incAction->setShortcut(KShortcut(_incShortcut));
+            _incAction->setShortcut(QKeySequence(_incShortcut));
         }
 
         if(_decShortcut != controller()->decreaseShortcut()) {
             _decShortcut = controller()->decreaseShortcut();
-            _decAction->setShortcut(KShortcut(_decShortcut));
+            _decAction->setShortcut(QKeySequence(_decShortcut));
         }
 
         //if(!graph()->autoLimitsX() && !graph()->autoLimitsY()) adjustLimits();
@@ -1587,10 +1627,10 @@ void ControllerMenuHandler::populateMenu(QMenu* menu, KActionCollection* actions
     _confDialog = 0;
     _confChanged = false;
 
-    menu->addAction(KIcon("arrow-up"), i18n("Increase value"), this, SLOT(incTriggered()));
-    menu->addAction(KIcon("arrow-down"), i18n("Decrease value"), this, SLOT(decTriggered()));
+    menu->addAction(QIcon::fromTheme("arrow-up"), i18n("Increase value"), this, SLOT(incTriggered()));
+    menu->addAction(QIcon::fromTheme("arrow-down"), i18n("Decrease value"), this, SLOT(decTriggered()));
     menu->addSeparator();
-    menu->addAction(KIcon("configure"), i18n("Configure controller..."), this, SLOT(configureController()));
+    menu->addAction(QIcon::fromTheme("configure"), i18n("Configure controller..."), this, SLOT(configureController()));
     menu->addSeparator();
     ItemMenuHandler::populateMenu(menu, actions);
 }
@@ -1606,13 +1646,26 @@ void ControllerMenuHandler::configureController()
         _worldModel->simulationStop();
 
     _confChanged = false;
-    _confDialog = new KDialog(); // XXX
+    _confDialog = new QDialog(); // XXX
     
-    _confDialog->setCaption(i18n("Configure controller"));
-    _confDialog->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
+    _confDialog->setWindowTitle(i18n("Configure controller"));
+    _buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
+				      | QDialogButtonBox::Cancel
+				      | QDialogButtonBox::Apply);
+    QWidget *mainWidget = new QWidget();
+    QVBoxLayout *mainLayout = new QVBoxLayout;
+    _confDialog->setLayout(mainLayout);
+    mainLayout->addWidget(mainWidget);
+
+    QPushButton *okButton = _buttonBox->button(QDialogButtonBox::Ok);
+    okButton->setDefault(true);
+    okButton->setShortcut(Qt::CTRL | Qt::Key_Return);
+    _confDialog->connect(_buttonBox, SIGNAL(accepted()), _confDialog, SLOT(accept()));
+    _confDialog->connect(_buttonBox, SIGNAL(rejected()), _confDialog, SLOT(reject()));
+    mainLayout->addWidget(_buttonBox);
 
     _confUi = new Ui::WidgetConfigureController;
-    _confUi->setupUi(_confDialog->mainWidget());
+    _confUi->setupUi(mainWidget);
 
     _confUi->dataSource->setSkipReadOnly(true);
     _confUi->dataSource->setDataSource(_worldModel, controller()->object(),
@@ -1631,17 +1684,16 @@ void ControllerMenuHandler::configureController()
 
     //_confUi->keyIncrease->setKeySequence(_incAction->shortcut().primary());
     //_confUi->keyDecrease->setKeySequence(_decAction->shortcut().primary());
-    _confUi->keyIncrease->setKeySequence(KShortcut(controller()->increaseShortcut()).primary());
-    _confUi->keyDecrease->setKeySequence(KShortcut(controller()->decreaseShortcut()).primary());
+    _confUi->keyIncrease->setKeySequence(QKeySequence(controller()->increaseShortcut()));
+    _confUi->keyDecrease->setKeySequence(QKeySequence(controller()->decreaseShortcut()));
 
     _confUi->lineEditIncrement->setValidator(
                 new QDoubleValidator(-HUGE_VAL, HUGE_VAL, DBL_DIG, _confUi->lineEditIncrement));
     _confUi->lineEditIncrement->setText(QString::number(controller()->increment()));
 
-    _confDialog->enableButtonApply(false);
+    _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
 
-    connect(_confDialog, SIGNAL(applyClicked()), this, SLOT(confApply()));
-    connect(_confDialog, SIGNAL(okClicked()), this, SLOT(confApply()));
+    connect(_buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(confApply(QAbstractButton*)));
 
     connect(_confUi->dataSource, SIGNAL(dataSourceChanged()), this, SLOT(confChanged()));
     connect(_confUi->lineEditMin, SIGNAL(textEdited(const QString&)), this, SLOT(confChanged()));
@@ -1656,8 +1708,13 @@ void ControllerMenuHandler::configureController()
     delete _confUi; _confUi = 0;
 }
 
-void ControllerMenuHandler::confApply()
+void ControllerMenuHandler::confApply(QAbstractButton *button)
 {
+    if (_buttonBox->button(QDialogButtonBox::Apply) != button
+	&& _buttonBox->button(QDialogButtonBox::Ok) != button) {
+	return;
+    }
+
     Q_ASSERT(_confUi && _confDialog);
 
     // XXX: check for actual change ?
@@ -1693,7 +1750,7 @@ void ControllerMenuHandler::confChanged()
 {
     Q_ASSERT(_confUi && _confDialog);
     _confChanged = true;
-    _confDialog->enableButtonApply(true);
+    _buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
 }
 
 void ControllerMenuHandler::decTriggered()
@@ -1852,7 +1909,7 @@ void TracerGraphicsItem::mouseSetPos(const QPointF&, const QPointF& diff, Moving
 
 void TracerMenuHandler::populateMenu(QMenu* menu, KActionCollection* actions)
 {
-    menu->addAction(KIcon("edit-clear"), i18n("Clear trace"), this, SLOT(clearTracer()));
+    menu->addAction(QIcon::fromTheme("edit-clear"), i18n("Clear trace"), this, SLOT(clearTracer()));
     menu->addSeparator();
     ItemMenuHandler::populateMenu(menu, actions);
 }
